"variations":
{
	"point":
	{
	},
	"directional":
	{
		"default": true
	},
	"spot":
	{
	},
	"MSAA":
	{
		"sub_variation": true
	}
}

#version 450
#extension GL_ARB_separate_shader_objects : enable

// inputs
struct shader_input_0
{
	float2 texcoord : TEXCOORD0;
};

// outputs
struct shader_output_0
{
	float4 colour : COLOUR0;
};

// uniforms
MSAA:
{
	layout(binding = 0) uniform sampler2DMS gbuffer_0;
	layout(binding = 1) uniform sampler2DMS gbuffer_1;
	layout(binding = 2) uniform sampler2DMS gbuffer_2;
	layout(binding = 3) uniform sampler2DMS depth_buffer;
}
else:
{
	layout(binding = 0) uniform sampler2D gbuffer_0;
	layout(binding = 1) uniform sampler2D gbuffer_1;
	layout(binding = 2) uniform sampler2D gbuffer_2;
	layout(binding = 3) uniform sampler2D depth_buffer;
}

directional/spot:
{
	layout(binding = 4) uniform sampler2DShadow shadowMap;
}

point:
{
	layout(binding = 4) uniform samplerCubeShadow shadowMap;
}

layout(binding = 0) uniform GenericViewUniforms
{
	vec4 dims;
} view;

layout(binding = 1) uniform ViewUniforms
{
	mat4 inverseVP;
	vec4 eye;
} uniforms;

layout(binding = 3) uniform LightData
{
	mat4 world;
	mat4 view;
	mat4 projection;
	vec4 diffuse;
	vec4 user;
	
} light;

float toNonLinearDepth(float depth, float near, float far)
{
	float z = (far + near) / (far - near) - (2.0 * far * near) / (far - near) / depth;
	return z * 0.5 + 0.5;
}

void main()
{
	MSAA:
	{
		ivec2 texelCoord = ivec2(view.dims.xy * shader_input_0.texcoord);	
		float depth = texelFetch(depth_buffer, texelCoord, gl_SampleID).r;
	}
	else:
	{
		float depth = texture(depth_buffer, shader_input_0.texcoord).r;
	}
	
	if(depth >= 1.0)
	{
		discard;
	}
	
	vec4 pos = vec4(shader_input_0.texcoord, depth, 1.0);
	pos.xy = pos.xy * 2.0 - 1.0;
	
	pos = uniforms.inverseVP * pos;
	pos = pos / pos.w;
	
	MSAA:
	{
		vec4 normal = texelFetch(gbuffer_0, texelCoord, gl_SampleID);
		vec4 diffuse = texelFetch(gbuffer_1, texelCoord, gl_SampleID);
		vec4 specular = texelFetch(gbuffer_2, texelCoord, gl_SampleID);
	}
	else:
	{
		vec4 normal = texture(gbuffer_0, shader_input_0.texcoord);
		vec4 diffuse = texture(gbuffer_1, shader_input_0.texcoord);
		vec4 specular = texture(gbuffer_2, shader_input_0.texcoord);
	}
	
	vec4 colour = vec4(0.0, 0.0, 0.0, 1.0);
	
	float lightStrength = 0.0f;
	point/spot:
	{
		// calculate fragment to light ray
		vec4 lightPos = light.world * vec4(0.0, 0.0, 0.0, 1.0);
		lightPos /= lightPos.w;
		
		vec3 lightRay = lightPos.xyz - pos.xyz;
		float falloff = 1.0 - (length(lightRay) / light.user.x);
		lightRay = normalize(lightRay);
		lightStrength = falloff * clamp(dot(lightRay, normal.xyz), 0.0, 1.0);
	}
	
	point:
	{
		// calculate shadow occlusion
		vec3 lightToPos = pos.xyz - lightPos.xyz;
		vec3 absVec = abs(lightToPos);
		float localZ = max(absVec.x, max(absVec.y, absVec.z));
		
		float near = light.user.z;
		float far = light.user.w;
		float z = (far + near) / (far - near) - (2.0 * far * near) / (far - near) / localZ;
		z = z * 0.5 + 0.5;
		
		vec3 shadowmapTexcoord = normalize(lightToPos);
		float occlusion = texture(shadowMap, vec4(shadowmapTexcoord.xyz, z - 0.00001));
		
		lightStrength = lightStrength * occlusion;
	}
	
	spot:
	{
		// calculate shadow occlusion
		float fragDepth = length(pos.xyz - lightPos.xyz);
		float z = toNonLinearDepth(fragDepth, 0.1, light.user.z);
		
		vec4 shadowmapTexcoord = light.projection * light.view * vec4(pos.xyz, 1.0);
		shadowmapTexcoord /= shadowmapTexcoord.w;
		shadowmapTexcoord.x = shadowmapTexcoord.x * 0.5 + 0.5;
		shadowmapTexcoord.y = shadowmapTexcoord.y * 0.5 + 0.5;
		float occlusion = texture(shadowMap, vec3(shadowmapTexcoord.xy, shadowmapTexcoord.z - 0.00001));
		
		// calculate spot falloff
		lightRay = -lightRay;
		
		vec3 lightDir = mat3(light.world) * vec3(0.0, 0.0, 1.0);
		lightDir = normalize(lightDir);
		
		float spotFalloff = smoothstep(light.user.y, 1.0, clamp(dot(lightDir, lightRay), 0.0, 1.0));
		lightStrength = lightStrength * spotFalloff * occlusion;
		
	}
	
	directional:
	{
		// calculate shadow occlusion
		vec4 projected = light.projection * light.view * vec4(pos.xyz, 1.0);
		projected /= projected.w;
		
		projected.x = projected.x * 0.5 + 0.5;
		projected.y = projected.y * 0.5 + 0.5;
		float occlusion = texture(shadowMap, vec3(projected.xy, projected.z - 0.001));
		
		vec3 lightRay = mat3(light.world) * vec3(0.0, 0.0, 1.0);
		lightRay = normalize(lightRay);
		lightRay = -lightRay;
		
		lightStrength = clamp(dot(lightRay, normal.xyz), 0.0, 1.0) * occlusion;
	}
	
	// calculate specular component
	vec3 specularColour = vec3(0.0, 0.0, 0.0);
	if(specular.a > 0.0)
	{
		vec3 viewRay = normalize(uniforms.eye.xyz - pos.xyz);
		vec3 reflected = reflect(-lightRay.xyz, normal.xyz);
		float specularPower = pow(clamp(dot(viewRay, reflected), 0.0, 1.0), specular.a * 255.0);
		specularColour = specular.xyz * light.diffuse.xyz * specularPower;
	}
	colour.xyz = diffuse.xyz * (light.diffuse.xyz * lightStrength) + (specularColour.xyz * lightStrength);
	
	shader_output_0.colour = colour;
}
