"variations":
{
	"forward":
	{
		"default": true
	},
	"gbuffer":
	{
		"blend_state":
		{
			"enabled": false
		}
	},
	"shadow_map":
	{
	},
	"diffuse_map":
	{
		"sub_variation": true
	},
	"specular_map":
	{
		"sub_variation": true
	},
	"bump_map":
	{
		"sub_variation": true
	},
	"alpha_map":
	{
		"sub_variation": true
	}
}

#version 450
#extension GL_ARB_separate_shader_objects : enable

// inputs
forward/shadow_map:
{
	struct shader_input_0
	{
		float2 texcoord : TEXCOORD0;
		float3 normal : NORMAL0;
	};
}

gbuffer:
{
	struct shader_input_0
	{
		float3 world_pos : POSITION1;
		float2 texcoord : TEXCOORD0;
		float3 normal : NORMAL0;
	};
}

// outputs
forward:
{
	struct shader_output_0
	{
		float4 colour : COLOUR0;
	};
}

gbuffer:
{
	struct shader_output_0
	{
		float4 normal : COLOUR0;
		float4 diffuse : COLOUR1;
		float4 specular : COLOUR2;
	};
}


// uniforms
diffuse_map:
{
	layout(binding = 0) uniform sampler2D diffuse;
}

specular_map:
{
	layout(binding = 1) uniform sampler2D specular;
}

bump_map:
{
	layout(binding = 2) uniform sampler2D bump;
}

alpha_map:
{
	layout(binding = 3) uniform sampler2D alpha;
}

layout(binding = 4) uniform MaterialData
{
	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
	float shininess;
	float ior;
	float dissolve;
	float illum;
} material;

void main()
{

	vec4 colour = material.diffuse;
	
	colour.a = colour.a * material.dissolve;
	alpha_map:
	{
		colour.a = colour.a * texture(alpha, shader_input_0.texcoord).r;
	}
	
	bool readMaps = true;
	
	shadow_map:
	{
		if(colour.a < 1.0)
		{
			discard;
		}
		
		readMaps = false;
	}
	
	
	vec3 normal = shader_input_0.normal;
	vec4 spec = material.specular;
	
	if(readMaps == true)
	{
		diffuse_map:
		{
			colour = colour * texture(diffuse, shader_input_0.texcoord);
		}
		
		bump_map:
		{
			vec3 offset = texture(bump, shader_input_0.texcoord).xyz;
			offset = offset * 2.0 - 1.0;
			normal = normal + offset;
		}
		
		specular_map:
		{
			spec = spec * texture(specular, shader_input_0.texcoord);
		}
	}
	
	forward:
	{
		shader_output_0.colour = colour;
	}
		
	gbuffer:
	{
		if(colour.a < 1.0)
		{
			discard;
		}
		
		shader_output_0.diffuse = colour;
		shader_output_0.normal = vec4(normal, 1.0);
		shader_output_0.specular = vec4(spec.xyz, material.shininess / 255.0);
	}	
}