#pragma once

#include "codegen/animations.h"

#include "world.h"

#include "json.h"

namespace engine
{
	struct AnimMapping
	{
		ID		m_id[9];
		ID		m_targetAlias[9];
		size_t	m_targetOffset[9];
		size_t	m_groupIndex[9];
		ID		m_groupAlias[9];
		size_t	m_numGroups = 0;
	};

	struct AnimData
	{
		float	m_floats[9];
	};

	class AnimWorld : public WorldBase
	{
	public:
		COMPONENT_ARRAY(ID, 			id);
		COMPONENT_ARRAY(size_t,			targetIndex);
		COMPONENT_ARRAY(AnimMapping,	mapping);
		COMPONENT_ARRAY(size_t,			dataCount);
		COMPONENT_ARRAY(AnimData,       data);
		COMPONENT_ARRAY(AnimLoopType,	loopType);
		COMPONENT_ARRAY(AnimInterpType, interpType);
		COMPONENT_ARRAY(AnimDriverType, driverType);
		COMPONENT_ARRAY(float,			time);
		COMPONENT_ARRAY(float, 			duration);
		COMPONENT_ARRAY(float, 			speed);
		COMPONENT_ARRAY(ID,				driverTarget);
		COMPONENT_ARRAY(size_t,			driverIndex);
		COMPONENT_ARRAY(size_t,			driverAlias);

		AnimWorld()
		{
			m_last = &m_driverAlias;
		}

		void	destroyEntity(size_t e) override;
		u64		enumNameToComponentFlagValue(const std::string& name) override;
		u64		enumNameToEntityFlagValue(const std::string& name) override;
	};

	class AnimationManager
	{
	public:
		void		setup(World* w);
		void		shutdown();
		void		update(float dt);

		void		addAnimation(size_t targetIndex, ID animID, const rapidjson::Value& anim);

		AnimWorld& getWorld() { return m_animWorld; }

	protected:
		World*					m_world = nullptr;
		AnimWorld			    m_animWorld;
	};
}