#pragma once

#include "defs.h"
#include "renderer.h"
#include "scene.h"
#include "logic.h"

#include <atomic>

namespace engine
{
    typedef Scene* (*SceneCreateFunc)(void*);

    struct SceneDesc
    {
        ID          m_funcID = 0;
        std::string m_filename = "";
    };

    struct EngineDesc
    {

    };

    class Engine
    {
    public:
        virtual void        setup() = 0;
        virtual void        shutdown() = 0;
        virtual void        processInput(std::vector<utils::InputEvent>& events) = 0;
        virtual void        update(float dt) = 0;
        virtual void        render() = 0;
        virtual void        resize(u32 width, u32 height) = 0;
        virtual void        preloadScene(const SceneDesc& desc) = 0;
        virtual void        loadScene(const SceneDesc& desc) = 0;
                
        virtual Scene*      getCurrentScene() = 0;

        Scene*              createScene(const SceneDesc& desc);
        TriggerCondition*   createTriggerCondition(ID id, const rapidjson::Value& j, Scene* s, size_t index = -1);
        TriggerAction*      createTriggerAction(ID id, const rapidjson::Value& j, Scene* s, size_t index = -1);

        void                registerTriggerCondition(TriggerCondition* condition);
        void                registerTriggerAction(TriggerAction* action);
        
        GameState&          getGameState() { return m_gameState; }
        virtual Renderer&   getRenderer() = 0;

        // functions for registering external functionality
        void registerSceneCreateFunc(SceneCreateFunc func, const std::string& funcName)
        {
            ID id = hash(funcName);
            m_sceneCreateFuncs[id] = func;
        }


        static Engine* get()
        {
            return m_self;
        }

    protected:        
        std::unordered_map<ID, SceneCreateFunc>     m_sceneCreateFuncs;
        std::unordered_map<ID, TriggerCondition*>   m_triggerConditions;
        std::unordered_map<ID, TriggerAction*>      m_triggerActions;
        GameState                                   m_gameState;

        static Engine* m_self;
    };

    Engine* createEngine(const EngineDesc& desc);
}
