#pragma once

#include "defs.h"

#include <string>
#include <rapidjson/document.h>

namespace engine
{
    template <typename T>
    T enumNameToValue(const char** names, size_t count, const std::string& name)
    {
        for (int i = 0; i < count; ++i)
        {
            if (name == names[i])
            {
                return (T)i;
            }
        }
        return (T)0;
    }

    u64 enumNameToFlagValue(const char** names, size_t count, const std::string& name);

    template <typename T>
    T getJsonValueOrDefault(const rapidjson::Value& json, std::string key, T defaultValue)
    {
        if (json.HasMember(key.c_str()))
        {
            return json[key.c_str()].Get<T>();
        }
        return defaultValue;
    }

    template <typename T>
    void getJsonArrayOrDefault(const rapidjson::Value& json, std::string key, T* data, size_t count, T defaultValue)
    {
        size_t lastNonDefault = 0;
        if (json.HasMember(key.c_str()) && json[key.c_str()].IsArray())
        {
            const rapidjson::Value& val = json[key.c_str()];
            for (size_t i = 0; i < val.Size(); ++i)
            {
                data[i] = val[i].Get<T>();
            }
            lastNonDefault = val.Size();
        }

        // complete array with default values
        for (size_t i = lastNonDefault; i < count; ++i) 
        {
            data[i] = defaultValue;
        }
    }

    bool mergeObjects(rapidjson::Value& dst, const rapidjson::Value& src, rapidjson::Document::AllocatorType& allocator);
}