#pragma once

#include "defs.h"
#include "renderer.h"
#include "resources.h"

#include <glm/glm.hpp>
#include <tiny_obj_loader.h>

namespace engine
{
    class World;

    struct VertexTextured
    {
        glm::vec4 m_position;
        glm::vec2 m_texcoord;
    };

    struct VertexTexturedNormal
    {
        glm::vec4 m_position;
        glm::vec2 m_texcoord;
        glm::vec3 m_normal;
    };

    struct MaterialLoadDesc
    {
        std::string                 m_filename = "";
        tinyobj::material_t         m_params;
        std::vector<std::string>    m_textureFilenames;
        ID                          m_defaultSamplerID = 0;
    };

    struct MaterialData
    {
        std::vector<TextureRef> m_textures;
        gfx::Buffer             m_buffer = nullptr;
        u32                     m_flags = 0;
        ID                      m_defaultSamplerID = 0;
    };
    typedef ResourceRef<MaterialData> MaterialRef;

    struct GeometryCpuData
    {
        Mem         m_vertexData;
        Mem         m_indexData;
        u32         m_numVertices = 0;
        u32         m_numIndices = 0;
        float       m_minVert[4] = { 0.0f, 0.0f, 0.0f, 0.0f };
        float       m_maxVert[4] = { 0.0f, 0.0f, 0.0f, 0.0f };
    };

    struct ModelAsyncData
    {
        std::vector<GeometryCpuData> m_cpuBuffers;
        std::vector<MaterialRef>     m_materials;
    };

    typedef ModelAsyncData (*GeometryGenFunc)(void*);

    struct ModelLoadDesc
    {
        std::string     m_filename = "";
        GeometryGenFunc m_genFunc = nullptr;
        Mem             m_genData;
    };

    struct ModelData
    {
        std::vector<gfx::Geometry>  m_meshes;
        std::vector<MaterialRef>    m_materials;
        std::vector<size_t>         m_materialIndices;
        size_t                      m_numUniques = 0;
    };
    typedef ResourceRef<ModelData> ModelRef;

    struct PlaneGenData
    {
        float	m_offset[3] = { 0.0f, 0.0f, 0.0f };
        float   m_normal[3] = { 0.0f, 0.0f, 0.0f };
        float	m_subdivisions = 1.0f;
    };

    // functions for creating geometry
    ModelAsyncData generatePlaneGeometry(void* data);
    ModelAsyncData generateCubeGeometry(void* data);
}
