#include "json.h"

namespace engine
{
    u64 enumNameToFlagValue(const char** names, size_t count, const std::string& name)
    {
        for (int i = 0; i < count; ++i)
        {
            if (name == names[i])
            {
                return 1 << i;
            }
        }
        return 0;
    }

    bool mergeObjects(rapidjson::Value& dst, const rapidjson::Value& src, rapidjson::Document::AllocatorType& allocator)
    {
        for (auto srcIt = src.MemberBegin(); srcIt != src.MemberEnd(); ++srcIt)
        {
            auto dstIt = dst.FindMember(srcIt->name);
            if (dstIt == dst.MemberEnd())
            {
                rapidjson::Value dstName;
                dstName.CopyFrom(srcIt->name, allocator);
                rapidjson::Value dstValue;
                dstValue.CopyFrom(srcIt->value, allocator);

                dst.AddMember(dstName, dstValue, allocator);
            }
            else
            {
                auto srcType = srcIt->value.GetType();
                auto dstType = dstIt->value.GetType();
                if (srcType != dstType)
                    return false;

                if (srcIt->value.IsArray())
                {
                    for (auto arrayIt = srcIt->value.Begin(); arrayIt != srcIt->value.End(); ++arrayIt)
                    {
                        rapidjson::Value dstValue;
                        dstValue.CopyFrom(*arrayIt, allocator);
                        dstIt->value.PushBack(dstValue, allocator);
                    }
                }
                else if (srcIt->value.IsObject())
                {
                    if (!mergeObjects(dstIt->value, srcIt->value, allocator))
                        return false;
                }
                else
                {
                    dstIt->value.CopyFrom(srcIt->value, allocator);
                }
            }
        }
        return true;
    }
}